import secrets
import tempfile
from pathlib import Path
from typing import List, Union
from urllib.parse import quote, urlencode

import pandas as pd
import requests

from astromodule.io import (batch_download_file, download_file, load_table,
                            save_table)
from astromodule.tableops import concat_tables


class TapService:
  def __init__(self, url):
    self.url = url
    self.http_client = requests.Session()

  def sync_query(self, query: str, save_path: Union[str, Path]):
    params = {
      'request': 'doQuery',
      'version': 1.0,
      'lang': 'ADQL',
      'phase': 'run',
      'format': 'csv',
      'query': query
    }
    req_url = self.url + '?' + urlencode(params)
    save_path = Path(save_path)
    save_path.parent.mkdir(parents=True, exist_ok=True)
    download_file(req_url, save_path, replace=True, http_client=self.http_client)

  def batch_sync_query(
    self,
    queries: List[str],
    save_paths: List[str] | List[Path] | str | Path,
    join_outputs: bool = False,
    workers: int = 3
  ):
    params = {
      'request': 'doQuery',
      'version': 1.0,
      'lang': 'ADQL',
      'phase': 'run',
      'format': 'csv'
    }
    urls = [self.url + '?' + urlencode({**params, 'query': q}) for q in queries]

    save_paths_aux = save_paths
    if join_outputs:
      tmp_folder = Path(tempfile.gettempdir()) / f'tap_{secrets.token_hex(3)}'
      tmp_folder.mkdir(parents=True)
      save_paths_aux = [tmp_folder / f'{i}.csv' for i in range(len(queries))]

    batch_download_file(
      urls,
      save_path=save_paths_aux,
      http_client=self.http_client,
      workers=workers
    )

    if join_outputs:
      combined_df = concat_tables(tmp_folder.glob('*.csv'), comment='#')
      save_table(combined_df, save_paths)





if __name__ == '__main__':
  tap = TapService('https://datalab.noirlab.edu/tap/sync')
  tap.sync_query('select top 10 psfdepth_g, psfdepth_r from ls_dr9.tractor where ra between 230.2939-0.0013 and 230.2939+0.0013 and dec between 29.7714-0.0013 and 29.7714+0.0013')
  tap.sync_query('select top 10 psfdepth_g, psfdepth_r from ls_dr9.tractor where ra between 230.2939-0.0013 and 230.2939+0.0013 and dec between 29.7714-0.0013 and 29.7714+0.0013')



import re
import secrets
import subprocess
import tempfile
from dataclasses import dataclass
from difflib import get_close_matches
from io import BytesIO, StringIO
from pathlib import Path
from typing import Literal, Sequence, Tuple, Union

import astropy.units as u
import numpy as np
import pandas as pd
from astropy.coordinates import SkyCoord, match_coordinates_sky
from astropy.table import Table

from astromodule.io import load_table, save_table

RA_REGEX = re.compile(r'^ra_?\d*$', re.IGNORECASE)
DEC_REGEX = re.compile(r'^dec_?\d*$', re.IGNORECASE)

def _match_regex_against_sequence(
  regex: re.Pattern, 
  columns: Sequence[str]
) -> Tuple[int, str] | None:
  for i, col in enumerate(columns):
    if regex.match(col):
      return i, col
  return None


def guess_coords_columns(
  df: pd.DataFrame,
  ra: str | None = None,
  dec: str | None = None,
) -> Tuple[str, str]:
  cols = df.columns.to_list()
  if ra is None:
    _, ra = _match_regex_against_sequence(RA_REGEX, cols)
  if dec is None:
    _, dec = _match_regex_against_sequence(DEC_REGEX, cols)
  if ra is None or dec is None:
    raise ValueError(
      "Can't guess RA or DEC columns, please, specify the columns names "
      "via `ra` and `dec` parameters"
    )
  return ra, dec



def table_knn(
  left: str | Path | pd.DataFrame | Table,
  right: str | Path | pd.DataFrame | Table,
  nthneighbor: int = 1,
  left_ra: str = 'ra',
  left_dec: str = 'dec',
  right_ra: str = 'ra',
  right_dec: str = 'dec',
) -> Tuple[np.ndarray, np.ndarray]:
  left_df = load_table(left)
  right_df = load_table(right)
  
  left_coords = SkyCoord(
    ra=left_df[left_ra].values,
    dec=left_df[left_dec].values, 
    unit=u.deg,
  )
  right_coords = SkyCoord(
    ra=right_df[right_ra].values,
    dec=right_df[right_dec].values,
    unit=u.deg,
  )
  
  idx, d, _ = match_coordinates_sky(
    left_coords,
    right_coords,
    nthneighbor=nthneighbor
  )

  return np.array(idx), np.array(d)


def crossmatch(
  left: str | Path | pd.DataFrame | Table,
  right: str | Path | pd.DataFrame | Table,
  radius: float | u.Quantity = 1*u.arcsec,
  join: Literal['inner', 'left'] = 'inner',
  nthneighbor: int = 1,
  left_ra: str | None = None,
  left_dec: str | None = None,
  left_columns: Sequence[str] | None = None,
  right_ra: str | None = None,
  right_dec: str | None = None,
  right_columns: Sequence[str] | None = None,
  include_sep: bool = True,
):
  left_df = load_table(left)
  left_ra, left_dec = guess_coords_columns(left_df, left_ra, left_dec)
  right_df = load_table(right)
  right_ra, right_dec = guess_coords_columns(right_df, right_ra, right_dec)
  
  idx, d = table_knn(
    left_df, 
    right_df, 
    nthneighbor=nthneighbor, 
    left_ra=left_ra,
    left_dec=left_dec,
    right_ra=right_ra,
    right_dec=right_dec,
  )
  
  if isinstance(radius, u.Quantity):
    radius = radius.to(u.deg).value
  else:
    radius = u.Quantity(radius, unit=u.arcsec).to(u.deg).value

  mask = d < radius

  left_idx = mask.nonzero()[0]
  right_idx = idx[mask]
  
  if left_columns is not None:
    left_df = left_df[left_columns].copy()
  if right_columns is not None:
    right_df = right_df[right_columns].copy()
  
  if join == 'inner':
    left_masked_df = left_df.iloc[left_idx]
    right_masked_df = right_df.iloc[right_idx]
    match_df = left_masked_df.copy(deep=True)
    for col in right_masked_df.columns.to_list():
      if not col in match_df.columns:
        match_df[col] = right_masked_df[col].to_numpy()
        # TODO: include a flag "replace" in this method to indicate if t2 must
        # replace or not t1 columns. This implementation consider replace=False.
    if include_sep:
      match_df['xmatch_sep'] = d[mask]
  elif join == 'left':
    right_masked_df = right_df.iloc[right_idx]
    cols = [col for col in right_masked_df.columns if col not in left_df.columns]
    match_df = left_df.copy(deep=True)
    match_df.loc[left_idx, cols] = right_masked_df[cols].values
    if include_sep:
      match_df.loc[left_idx, 'xmatch_sep'] = d[mask]
  return match_df

  # left_df_masked = left_df.iloc[primary_idx]
  # right_df_masked = right_df.iloc[secondary_idx]

  # left_df_subsample = left_df_masked[left_columns].copy() \
  #   if left_columns is not None else left_df_masked.copy()
  # right_df_subsample = right_df_masked[right_columns].copy() \
  #   if right_columns.columns is not None else right_df_masked.copy()

  # for col in right_df_subsample.columns.tolist():
  #   left_df_subsample[col] = right_df_subsample[col].to_numpy()
  #   # TODO: include a flag "replace" in this method to indicate if t2 must
  #   # replace or not t1 columns. This implementation consider replace=True.

  # r = CrossMatchResult()
  # r.distance = d[mask]
  # r.primary_idx = primary_idx
  # r.secondary_idx = secondary_idx
  # r.table = df1_subsample
  


@dataclass
class DropDuplicatesResult:
  df: pd.DataFrame
  distances: np.ndarray
  n_iterations: int
  drop_count: int

  
def drop_duplicates(
  table: str | Path | pd.DataFrame | Table,
  radius: float | u.Quantity = 1*u.arcsec,
  ra: str | None = None,
  dec: str | None = None,
  columns: Sequence[str] | None = None,
  max_iterations: int = 20,
) -> DropDuplicatesResult:
  if isinstance(radius, u.Quantity):
    radius = radius.to(u.deg).value
  else:
    radius = u.Quantity(radius, unit=u.arcsec).to(u.deg).value
  
  df = load_table(table)
  ra, dec = guess_coords_columns(df, ra, dec)
  df_coords = df[[ra, dec]].copy(deep=True)
  total_drop_count = 0
  drop_count = -1
  iteration = 0
  
  while (drop_count != 0 and iteration <= max_iterations):
    print(drop_count, iteration)
    idx, d = table_knn(
      df_coords, 
      df_coords, 
      left_ra=ra, 
      left_dec=dec, 
      right_ra=ra, 
      right_dec=dec, 
      nthneighbor=2
    )

    mask = d < radius
    primary_idx = mask.nonzero()[0]
    secondary_idx = idx[mask]
    removed_idx = []

    for pid, sid in zip(primary_idx, secondary_idx):
      if sid not in removed_idx:
        removed_idx.append(pid)

    del_mask = np.isin(idx, removed_idx, invert=True).nonzero()[0]
    len_copy_df = len(df_coords)
    df_coords = df_coords.iloc[del_mask].copy()
    
    drop_count = len_copy_df - len(df_coords)
    total_drop_count += drop_count
    iteration += 1
  
  d = d[del_mask]
  final_df = df.iloc[df_coords.index]
  if columns is not None:
    final_df = final_df[columns]
  
  return DropDuplicatesResult(
    df=final_df,
    distances=d,
    n_iterations=iteration,
    drop_count=total_drop_count
  )



  
def stilts_crossmatch(
  table1: pd.DataFrame | Table | str | Path,
  table2: pd.DataFrame | Table | str | Path,
  ra1: str | None = None,
  dec1: str | None = None,
  ra2: str | None = None,
  dec2: str | None = None,
  radius: float | u.Quantity = 1 * u.arcsec,
  join: Literal['1and2', '1or2', 'all1', 'all2', '1not2', '2not1', '1xor2'] = '1and2',
  find: Literal['all', 'best', 'best1', 'best2'] = 'best',
  fixcols: Literal['dups', 'all', 'none'] = 'dups',
  suffix1: str = '_1',
  suffix2: str = '_2',
  scorecol: str | None = 'xmatch_sep',
  fmt: Literal['fits', 'csv'] = 'fits',
) -> pd.DataFrame | None:
  tmpdir = Path(tempfile.gettempdir())
  token = secrets.token_hex(8)
  tb1_path = tmpdir / f'xmatch_in1_{token}.{fmt}'
  tb2_path = tmpdir / f'xmatch_in2_{token}.{fmt}'
  
  df1 = load_table(table1)
  df2 = load_table(table2)
  
  ra1, dec1 = guess_coords_columns(df1, ra1, dec1)
  ra2, dec2 = guess_coords_columns(df2, ra2, dec2)
  
  save_table(df1, tb1_path)
  save_table(df2, tb2_path)
  
  if isinstance(radius, u.Quantity):
    radius = int(radius.to(u.arcsec).value)
  else:
    radius = int(radius)
  
  cmd = [
    'stilts',
    'tmatch2',
    'matcher=sky',
    'progress=none',
    'runner=parallel',
    f'ifmt1={fmt}',
    f'ifmt2={fmt}',
    f'ofmt={fmt}',
    'omode=out',
    f'out=-',
    f'values1={ra1} {dec1}',
    f'values2={ra2} {dec2}',
    f'params={radius}',
    f'join={join}',
    f'find={find}',
    f'fixcols={fixcols}',
    f'suffix1={suffix1}',
    f'suffix2={suffix2}',
    f'scorecol={scorecol or "none"}',
    f'in1={str(tb1_path.absolute())}',
    f'in2={str(tb2_path.absolute())}',
  ]
  
  result = subprocess.run(
    cmd,
    stderr=subprocess.PIPE,
    stdout=subprocess.PIPE,
    text=False,
  )
  
  tb1_path.unlink()
  tb2_path.unlink()
  error = result.stderr.decode().strip()
  if error:
    print(error)
    return None
  
  df_out = load_table(BytesIO(result.stdout), fmt=fmt)
  return df_out



def stilts_unique(
  table: pd.DataFrame | Table | str | Path,
  radius: float | u.Quantity,
  action: Literal['identify', 'keep0', 'keep1', 'wide2', 'wideN'] = 'keep1',
  ra: str | None = None,
  dec: str | None = None,
  fmt: Literal['fits', 'csv'] = 'fits',
) -> pd.DataFrame | None:
  tmpdir = Path(tempfile.gettempdir())
  token = secrets.token_hex(8)
  in_path = tmpdir / f'xmatch_in_{token}.{fmt}'
  
  df = load_table(table)
  
  ra, dec = guess_coords_columns(df, ra, dec)
  
  save_table(df, in_path)
  
  if isinstance(radius, u.Quantity):
    radius = int(radius.to(u.arcsec).value)
  else:
    radius = int(radius)
    
  cmd = [
    'stilts',
    'tmatch1',
    'matcher=sky',
    f'params={radius}',
    f'values={ra} {dec}',
    f'action={action}',
    'progress=none',
    'runner=parallel',
    f'ifmt={fmt}',
    'omode=out',
    f'ofmt={fmt}',
    'out=-',
    f'in={str(in_path.absolute())}',
  ]
  
  result = subprocess.run(
    cmd,
    stderr=subprocess.PIPE,
    stdout=subprocess.PIPE,
    text=False,
  )
  
  in_path.unlink()
  error = result.stderr.decode().strip()
  if error:
    print(error)
    return None
  
  df_out = load_table(BytesIO(result.stdout), fmt=fmt)
  return df_out




def concat_tables(
  tables: Sequence[pd.DataFrame | str | Path | Table],
  **kwargs
) -> pd.DataFrame:
  dfs = [load_table(df, **kwargs) for df in tables]
  dfs = [df for df in dfs if isinstance(df, pd.DataFrame) and not df.empty]
  return pd.concat(dfs)



if __name__ == '__main__':
  df = stilts_unique(
    Path(__file__).parent.parent / 'tests' / 'selection_claudia+prepared.csv',
    radius=45*u.arcmin,
    action='indentify',
    fmt='csv'
  )
  print(df)



from io import BytesIO
from pathlib import Path
from typing import List, Sequence, Tuple, Union

import pandas as pd
import requests
from astropy import units as u
from astropy.table import Table

from astromodule.adql import AdqlColumn, AdqlDatabase, AdqlSchema, AdqlTable
from astromodule.io import (compress_fits, download_file, load_table,
                            parallel_function_executor)
from astromodule.tableops import guess_coords_columns
from astromodule.tap import TapService
from astromodule.utils import append_query_params, iauname, iauname_path

LEGACY_RGB_URL = 'https://www.legacysurvey.org/viewer/jpeg-cutout'
LEGACY_RGB_URL_DEV = 'https://www.legacysurvey.org/viewer-dev/jpeg-cutout'
LEGACY_FITS_URL = 'https://www.legacysurvey.org/viewer/fits-cutout'
LEGACY_FITS_URL_DEV = 'https://www.legacysurvey.org/viewer-dev/fits-cutout'
LEGACY_TAP_SYNC_DR10 = 'https://datalab.noirlab.edu/tap/sync'


# mag_r | pixscale
# 14-15 | 0.45
# 15-16 | 0.35
# 16-17 | 0.3
# 17-18 | 0.2


class LS10Tractor(AdqlTable):
  release = AdqlColumn('release')
  brickid = AdqlColumn('brickid')
  brickname = AdqlColumn('brickname')
  objid = AdqlColumn('objid')
  brick_primary = AdqlColumn('brick_primary')
  maskbits = AdqlColumn('maskbits')
  fitbits = AdqlColumn('fitbits')
  type = AdqlColumn('type')
  ra = AdqlColumn('ra')
  dec = AdqlColumn('dec')
  ra_ivar	= AdqlColumn('ra_ivar')
  dec_ivar = AdqlColumn('dec_ivar')
  bx = AdqlColumn('bx')
  by = AdqlColumn('by')
  nobs_g = AdqlColumn('nobs_g') 
  nobs_r = AdqlColumn('nobs_r')
  nobs_i = AdqlColumn('nobs_i')
  nobs_z = AdqlColumn('nobs_z')
  nobs_w1	= AdqlColumn('nobs_w1')
  nobs_w2	= AdqlColumn('nobs_w2')
  nobs_w3	= AdqlColumn('nobs_w3')
  nobs_w4	= AdqlColumn('nobs_w4')
  rchisq_g = AdqlColumn('rchisq_g')
  rchisq_r = AdqlColumn('rchisq_r')
  rchisq_i = AdqlColumn('rchisq_i')
  rchisq_z = AdqlColumn('rchisq_z')
  rchisq_w1 = AdqlColumn('rchisq_w1')
  rchisq_w2 = AdqlColumn('rchisq_w2')
  rchisq_w3 = AdqlColumn('rchisq_w3')
  rchisq_w4 = AdqlColumn('rchisq_w4')
  mag_g = AdqlColumn('mag_g')
  mag_r = AdqlColumn('mag_r')
  mag_i = AdqlColumn('mag_i')
  mag_z = AdqlColumn('mag_z')
  mag_w1 = AdqlColumn('mag_w1')
  mag_w2 = AdqlColumn('mag_w2')
  mag_w3 = AdqlColumn('mag_w3')
  mag_w4 = AdqlColumn('mag_w4')

class LS10(AdqlSchema):
  tractor = LS10Tractor()

class LegacyClass(AdqlDatabase):
  DR10 = LS10()


Legacy = LegacyClass()
Legacy.config()


class LegacyService(TapService):
  def __init__(
    self,
    replace: bool = False,
    width: float = 256,
    height: float = 256,
    pixscale: float = 0.27,
    bands: str = 'grz',
    layer: str = 'ls-dr10',
    use_dev: bool = False,
    fmt: str = 'jpg',
    compress_fits: bool = False,
    compress_type: str = 'HCOMPRESS_1',
    hcomp_scale: int = 3,
    quantize_level: int = 10,
    quantize_method: int = -1,
    workers: int = 3,
  ):
    """
    Parameters
    ----------
    replace: bool, optional
      Replace file if exists in ``save_path`` location
    width: float, optional
      Stamp width.
    height: float, optional
      Stamp height.
    pixscale: float, optional
      Pixel scale of the sky.
    bands: str, optional
      Image bands
    layer: str, optional
      Legacy Survey image layer.
    use_dev: bool, optional
      Use the dev env of Legacy Cutout API
    fmt: str, optional
      File format. One of: ``jpg`` or ``fits``
    compress_fits: bool, optional
      Compress the downloaded fits stamp to ``fits.fz``
    workers: int, optional
      Maximum spawned threads when `batch_cutout` is called
    """
    super().__init__(LEGACY_TAP_SYNC_DR10)
    self.replace = replace
    self.width = width
    self.height = height
    self.pixscale = pixscale
    self.bands = bands
    self.layer = layer
    self.use_dev = use_dev
    self.workers = workers
    self.compress_fits = compress_fits
    self.compress_type = compress_type
    self.hcomp_scale = hcomp_scale
    self.quantize_level = quantize_level
    self.quantize_method = quantize_method
    self.http_client = requests.Session()


  def cutout(
    self,
    ra: float,
    dec: float,
    save_path: Path = None,
    base_path: Union[str, Path] = '',
    mag_r: float = None,
  ) -> None:
    """
    Downloads a single Legacy Survey object RGB stamp defined by RA and DEC.

    Parameters
    ----------
    ra: float
      Right ascension of the object.
    dec: float
      Declination of the object.
    save_path: pathlib.Path, optional
      Path where downloaded file will be stored.
    base_path: str, pathlib.Path, optional
      The path that will be appended at beggining of every paths if ``save_path``
      is ``None``.
    """
    if self.image_format == 'jpg':
      url = LEGACY_RGB_URL_DEV if self.use_dev else LEGACY_RGB_URL
    else:
      url = LEGACY_FITS_URL_DEV if self.use_dev else LEGACY_FITS_URL

    if save_path is None:
      save_path = iauname_path(
        iaunames=iauname(ra=ra, dec=dec),
        prefix=Path(base_path),
        suffix=f'.{self.image_format}'
      )

    pixscale = self.get_pixscale(mag_r) if mag_r is not None else self.pixscale

    image_url = append_query_params(url, {
      'ra': ra,
      'dec': dec,
      'width': self.width,
      'height': self.height,
      'pixscale': pixscale,
      'bands': self.bands,
      'layer': self.layer
    })

    content = download_file(
      url=image_url,
      save_path=save_path,
      http_client=self.http_client,
      replace=self.replace,
      return_bytes=self.compress_fits,
    )

    if self.compress_fits:
      compress_fits(
        file=BytesIO(content),
        compress_type=self.compress_type,
        hcomp_scale=self.hcomp_scale,
        quantize_level=self.quantize_level,
        quantize_method=self.quantize_method,
        ext=0,
        save_path=save_path,
        replace=self.replace,
      )


  def batch_cutout(
    self,
    ra: List[float],
    dec: List[float],
    save_path: List[Path] = None,
    base_path: Union[str, Path] = '',
    mag_r: List[float] = None,
  ) -> Tuple[List[Path], List[Path]]:
    """
    Downloads a list of objects defined by RA and DEC coordinates.

    The ``ra``, ``dec`` and ``save_path`` lists are mandatory and
    must have same length.

    Parameters
    ----------
    ra: List[float]
      The list of RA coordinates of the desired objects.
    dec: List[float]
      The list of DEC coordinates of the desired objects.
    save_path: List[Path], optional
      The list of path where files should be saved.
    base_path: str, Path, optional
      The path that will be appended at beggining of every paths if ``save_path``
      is ``None``.
    """
    if save_path is None:
      save_path = iauname_path(
        iaunames=iauname(ra=ra, dec=dec),
        prefix=Path(base_path),
        suffix=f'.{self.image_format}'
      )

    if mag_r is None:
      params = [
        {
          'ra': _ra,
          'dec': _dec,
          'save_path': _save_path,
        }
        for _ra, _dec, _save_path in zip(ra, dec, save_path)
      ]
    else:
      params = [
        {
          'ra': _ra,
          'dec': _dec,
          'save_path': _save_path,
          'mag_r': _mag_r,
        }
        for _ra, _dec, _save_path, _mag_r in zip(ra, dec, save_path, mag_r)
      ]

    parallel_function_executor(
      self.cutout,
      params=params,
      workers=self.workers,
      unit='file'
    )

    success = [p for p in save_path if p.exists()]
    error = [p for p in save_path if not p.exists()]
    return success, error


  def get_pixscale(self, mag_r: Union[float, List[float]]) -> Union[float, List[float]]:
    if isinstance(mag_r, list):
      return [self.get_pixscale(m) for m in mag_r]

    if   14 <= mag_r < 15: return 0.45
    elif 15 <= mag_r < 16: return 0.35
    elif 16 <= mag_r < 17: return 0.3
    elif 17 <= mag_r < 18: return 0.2
    
    
  def crossmatch(
    self, 
    table: str | Path | pd.DataFrame | Table, 
    columns: Sequence[str] | Sequence[AdqlColumn], 
    radius: float | u.Quantity,
    save_path: str | Path,
    ra_col: str = None,
    dec_col: str = None,
    where: AdqlColumn = None,
    workers: int = 3,
  ):
    query_template = """
    SELECT TOP 1 {columns}, 
      POWER(ra - ({ra}), 2) + POWER(dec - ({dec}), 2) AS ls_separation
    FROM ls_dr10.tractor
    WHERE 
      ra BETWEEN {ra_min} AND {ra_max} AND 
      dec BETWEEN {dec_min} AND {dec_max} AND
      mag_r BETWEEN 10 AND 23
    ORDER BY ls_separation ASC
    """.strip()
    df = load_table(table)
    ra_col, dec_col = guess_coords_columns(df, ra_col, dec_col)
    tb = Legacy.DR10.tractor
    queries = []
    
    if isinstance(radius, u.Quantity):
      radius = radius.to(u.deg).value
    else:
      radius = (radius * u.arcsec).to(u.deg).value
      
    for i, row in df.iterrows():
      ra = row[ra_col]
      dec = row[dec_col]
      # match_cond = (
      #   tb.ra.between(ra - radius, ra + radius) & 
      #   tb.dec.between(dec - radius, dec + radius)
      # )
      # where = where & match_cond if where is not None else match_cond
      query = query_template.format(
        columns=','.join([str(c) for c in columns]), 
        ra_min=ra-radius,
        ra_max=ra+radius,
        dec_min=dec-radius,
        dec_max=dec+radius,
        ra=ra,
        dec=dec,
      )
      queries.append(query)
    self.batch_sync_query(
      queries=queries, 
      save_paths=save_path, 
      join_outputs=True, 
      workers=workers
    )
    


if __name__ == '__main__':
  # ls = LegacyService(workers=6)
  # ls.batch_download_legacy_rgb(
  #   ra=[185.1458 + dx/2 for dx in range(20)],
  #   dec=[12.8624 + dy/2 for dy in range(20)],
  #   save_path=[Path(f'test/{i}.jpg') for i in range(20)]
  # )
  
  ls = LegacyService()
  df = pd.DataFrame(({'ra':[184.4924, 184.4922], 'dec': [7.2737, 7.1862]}))
  ls.crossmatch(df, ['mag_r', 'mag_g', 'mag_i', 'mag_z'], 3, 'foba.csv')
